@@ -46,12 +46,14 @@ DTOutput = dataTableOutput
 #'   data is kept on the server and the browser requests a page at a time; if
 #'   \code{FALSE}, then the entire data frame is sent to the browser at once.
 #'   Highly recommended for medium to large data frames, which can cause
-#'   browsers to slow down or crash.
+#'   browsers to slow down or crash. Note that if you want to use
+#'   \code{renderDataTable} with \code{shiny::bindCache()}, this must be
+#'   \code{FALSE}.
 #' @param funcFilter (for expert use only) passed to the \code{filter} argument
 #'   of \code{\link{dataTableAjax}()}
 #' @param ... ignored when \code{expr} returns a table widget, and passed as
-#'   additional arguments to \code{\link{datatable}()} when \code{expr} returns a data
-#'   object
+#'   additional arguments to \code{\link{datatable}()} when \code{expr} returns
+#'   a data object
 renderDataTable = function(
     expr, server = TRUE, env = parent.frame(), quoted = FALSE,
     funcFilter = dataTablesFilter, ...
@@ -110,6 +112,11 @@ renderDataTable = function(
       # register the data object in a shiny session
       options = instance[['x']][['options']]
 
+      # autoHideNavigation won't work in the server mode
+      if (isTRUE(instance[['x']][['autoHideNavigation']]))
+        warning("`autoHideNavigation` only works with DT client mode and it will be ignored",
+                immediate. = TRUE, call. = FALSE)
+
       # Normalize "ajax" argument; if we leave it a string then we have several
       # code paths that need to account for both string and list representations
       if (is.character(options[['ajax']])) {
@@ -145,22 +152,39 @@ renderDataTable = function(
     widgetFunc(), dataTableOutput, environment(), FALSE
   )
 
-  func = shiny::markRenderFunction(dataTableOutput, function(shinysession, name, ...) {
-    domain = tempVarsPromiseDomain(outputInfoEnv, outputName = name, session = shinysession)
+  # The cacheHint arg is not present in Shiny < 1.6.0. Once that version is
+  # very widely used, we can remove this if() statement.
+  func = if ("cacheHint" %in% names(formals(shiny::markRenderFunction))) {
+    # Can't cache with server-side processing
+    cacheHint = if (server) FALSE else list(label = "renderDataTable", userExpr = expr)
 
-    promises::with_promise_domain(domain, renderFunc())
-  })
-
-  # This snapshotPreprocessOutput function was added in shiny 1.0.3.9002
-  if (exists("snapshotPreprocessOutput", asNamespace("shiny"))) {
-    func = shiny::snapshotPreprocessOutput(func, function(value) {
-      # Looks for a string like this in the JSON:
-      # "url":"session/2a2b834d90637a7559f3ebaba460ad10/dataobj/table?w=&nonce=aea032f33aedfd0e",
-      # and removes it, so that the value isn't saved in test snapshots.
-      gsub('"ajax"\\s*:\\s*\\{\\s*"url"\\s*:\\s*"[^"]*"\\s*,?', '"ajax":{', value)
-    })
+    shiny::markRenderFunction(
+      uiFunc = dataTableOutput,
+      renderFunc = function(shinysession, name, ...) {
+        domain = tempVarsPromiseDomain(outputInfoEnv, outputName = name, session = shinysession)
+        removeTimestampFromSnapshot(name)
+        promises::with_promise_domain(domain, renderFunc())
+      },
+      cacheHint = cacheHint
+    )
+  } else {
+    shiny::markRenderFunction(
+      uiFunc = dataTableOutput,
+      renderFunc = function(shinysession, name, ...) {
+        domain = tempVarsPromiseDomain(outputInfoEnv, outputName = name, session = shinysession)
+        removeTimestampFromSnapshot(name)
+        promises::with_promise_domain(domain, renderFunc())
+      }
+    )
   }
 
+  func = shiny::snapshotPreprocessOutput(func, function(value) {
+    # Looks for a string like this in the JSON:
+    # "url":"session/2a2b834d90637a7559f3ebaba460ad10/dataobj/table?w=&nonce=aea032f33aedfd0e",
+    # and removes it, so that the value isn't saved in test snapshots.
+    gsub('"ajax"\\s*:\\s*\\{\\s*"url"\\s*:\\s*"[^"]*"\\s*,?', '"ajax":{', value)
+  })
+
   shiny::registerInputHandler('DT.cellInfo', function(val, ...) {
     opts = options(stringsAsFactors = FALSE); on.exit(options(opts), add = TRUE)
     val = lapply(val, as.data.frame)
@@ -185,6 +209,13 @@ setAll = function(lst, env) {
   invisible()
 }
 
+removeTimestampFromSnapshot = function(name) {
+  shiny::snapshotPreprocessInput(paste0(name, "_state"), function(value) {
+    value$time <- NULL
+    value
+  })
+}
+
 # This promise domain is needed to set/unset temporary variables in
 # a specific environment anytime a promise handler is invoked in the
 # domain. This is used to pass the Shiny output name from where we
@@ -310,10 +341,11 @@ selectCells = function(proxy, selected, ignore.selectable = FALSE) {
 addRow = function(proxy, data, resetPaging = TRUE) {
   if ((is.matrix(data) || is.data.frame(data)) && nrow(data) != 1)
     stop("'data' must be of only one row")
+  rn <- rownames(data); if (!is.null(rn)) rn <- I(rn)
   # must apply unname() after as.list() because a data.table object
   # can't be really unnamed. The names() attributes will be
   # preserved but with empty strings (see #760).
-  invokeRemote(proxy, 'addRow', list(unname(as.list(data)), I(rownames(data)), resetPaging))
+  invokeRemote(proxy, 'addRow', list(unname(as.list(data)), rn, resetPaging))
 }
 
 #' @rdname proxy
@@ -432,13 +464,40 @@ reloadData = function(
 #'   enabled column filters, you should also make sure the attributes of every
 #'   column remain the same, e.g. factor columns should have the same or fewer
 #'   levels, and numeric columns should have the same or smaller range,
-#'   otherwise the filters may never be able to reach certain rows in the data.
+#'   otherwise the filters may never be able to reach certain rows in the data,
+#'   unless you explicitly update the filters with \code{updateFilters()}.
 #' @export
 replaceData = function(proxy, data, ..., resetPaging = TRUE, clearSelection = 'all') {
   dataTableAjax(proxy$session, data, ..., outputId = proxy$rawId)
   reloadData(proxy, resetPaging, clearSelection)
 }
 
+#' @rdname replaceData
+#' @export
+updateFilters = function(proxy, data) {
+  # calculate the values to be supplied to the filters based on column type
+  new_lims = lapply(data, function(x) {
+    if (inherits(x, c('numeric'))) {
+      range(x)
+    } else if (inherits(x, c('factor', 'logical'))) {
+      as.character(unique(x))
+    } else if (inherits(x, c('Date'))) {
+      as.numeric(as.POSIXct.Date(range(x))) * 100
+    } else if (inherits(x, 'POSIXt')) {
+      round(as.numeric(range(x)), digits = 2) * 100000
+    } else {
+      stop('updateFilters() requires all columns to be one of the following classes: numeric, factor, logical, Date, POSIXt')
+    }
+  })
+
+  # as of DT 0.19, numeric values fed to the sliders need to be multiplied by
+  # 10; e.g. 5.7 will be converted to 57
+  new_lims = unname(lapply(new_lims, function(x) if (is.numeric(x)) x * 10 else x))
+
+  # Trigger the JavaScript to update the filters
+  invokeRemote(proxy, 'updateFilters', list(new_lims))
+}
+
 invokeRemote = function(proxy, method, args = list()) {
   if (!inherits(proxy, 'dataTableProxy'))
     stop('Invalid proxy argument; table proxy object was expected')
@@ -519,14 +578,15 @@ dataTableAjax = function(session, data, rownames, filter = dataTablesFilter, out
 
 sessionDataURL = function(session, data, id, filter) {
 
-  URLdecode = shinyFun('URLdecode')
   toJSON = shinyFun('toJSON')
   httpResponse = shinyFun('httpResponse')
 
   filterFun = function(data, req) {
     # DataTables requests were sent via POST
-    params = URLdecode(rawToChar(req$rook.input$read()))
+    params = rawToChar(req$rook.input$read())
+    # I don't think the browser would send out nonASCII strings, but keep it as it is
     Encoding(params) = 'UTF-8'
+    # shiny::parseQueryString() calls httpuv::decodeURIComponent() internally and will handle encoding correctly
     params = shiny::parseQueryString(params, nested = TRUE)
 
     res = tryCatch(filter(data, params), error = function(e) {
